from django.db import models
from common_models import DateAware
from django.contrib.auth.models import User
#from lists import dataTypes#, dataSetups, dataClasses

#Note: what fields can be null

class Team(DateAware):
	name = models.CharField(max_length=200)
	number = models.IntegerField(primary_key=True)
	rookieYear = models.IntegerField(null=True)
	admin = models.ForeignKey(User, null=True)

	def __unicode__(self):
		return "Team #{}: {}".format(self.number, self.name)

# Create your models here.
class UserProfile(models.Model):
	team = models.ForeignKey(Team)
	teamRole = models.CharField(max_length=200, null=True)
	dateJoined = models.DateTimeField(null=True)
	user = models.OneToOneField(User)

	def __unicode__(self):
		return self.user.username

class GeneralTeamData(DateAware):
	commentor = models.ForeignKey(Team, related_name="commenterGeneralTeamData")
	about = models.ForeignKey(Team, related_name="aboutGeneralTeamData")
	comments = models.TextField()

	def __unicode__(self):
		return 'Data about {} by {}'.format(self.about.name, self.commentor.name) 

class Competition(DateAware):
	name = models.CharField(max_length = 200)
	year = models.IntegerField()

	def __unicode__(self):
		return '{}: {}'.format(self.year, self.name)

class Regional(DateAware):
	competition = models.ForeignKey(Competition)
	name = models.CharField(max_length = 200)
	place = models.CharField(max_length = 300)
	startDate = models.DateField()
	endDate = models.DateField()

	#TODO: add in place
	def __unicode__(self):
		return "{}".format(self.competition.name)

class Robot(DateAware):
	team = models.ForeignKey(Team)
	competition = models.ForeignKey(Competition)
	name = models.CharField(max_length=200)
	regionals = models.ManyToManyField(Regional,null=True)
	#description = models.TextField()   #who defines this? there won't always be someone on the team to do this

	def __unicode__(self):
		return '{}: Team #{}:{}, Competition:{}'.format(self.name, self.team.number, self.team.name, self.competition.name)

class TeamDataSetup(DateAware):
	creator = models.ForeignKey(Team)
	competition = models.ForeignKey(Competition)
	#permissions

	def __unicode__(self):
		return "{}'s for {}".format(self.creator.name, self.competition.name)

class Match(DateAware):
	dataSetup = models.ForeignKey(TeamDataSetup)
	matchNumber = models.IntegerField()
	regional = models.ForeignKey(Regional)
	matchType = models.IntegerField(null=True) #0 = practice, 1 = qualifier, 2 = elimination

	def __unicode__(self):
		"Match #{} at regional".format(self.matchNumber)

class Alliance(DateAware):
	match = models.ForeignKey(Match)
	color = models.IntegerField() #0 = red, 1 = blue
	robots = models.ManyToManyField(Robot, null=True)
	winner = models.BooleanField()
	score = models.IntegerField(null=True)
	penalties = models.IntegerField(null=True)

	def __unicode__(self):
		return '{}, Alliance {}'.format(self.match.__unicode__(), self.color)

class MatchRobot(DateAware):
	robot = models.ForeignKey(Robot)
	match = models.ForeignKey(Match)

	def __unicode__(self):
		return '{}, Robot: {}'.format(self.match.__unicode__(), self.robot.team.number)

class DataGroup(DateAware):
	name = models.CharField(max_length=200)
	description = models.TextField(null=True)
	teamDataSetup = models.ForeignKey(TeamDataSetup)

	def __unicode__(self):
		return '{} for Team DataSetup: {}'.format(self.name, self.teamDataSetup.__unicode__())


################################################
#Data Setup classes
class DataSetup(DateAware):
	dataGroup = models.ForeignKey(DataGroup)
	typeOfData = models.CharField(max_length=20)
	longName = models.CharField(max_length=200)
	shortName = models.CharField(max_length = 30)
	desc = models.TextField(null=True)

	def parseCommonForm(self, user, formData, number, groups, teamDataSetup):
		self.longName = formData['{}-longName'.format(number)]
		self.shortName = formData['{}-shortName'.format(number)]
		self.typeOfData = formData['{}-type'.format(number)]
		self.objectCreator = user
		#desc = formData['{}-desc'.format(number)]
		groupName = formData['{}-group'.format(number)]
		if groupName in groups:
			self.dataGroup = groups[groupName]
		else:
			group = DataGroup(name = groupName, teamDataSetup = teamDataSetup)
			group.save()
			print "group id", group.id
			self.dataGroup = group
			groups[groupName] = group

	def __unicode__(self):
		return 'Setup {} in the group {}'.format(self.longName, self.dataGroup.__unicode__()) 

class IncrementorSetup(DataSetup):
	startValue = models.IntegerField()
	incAmount = models.IntegerField()

	@classmethod
	def createFromForm(cls, user, formData, number, groups, teamDataSetup):
		startValue = formData['{}-startValue'.format(number)]
		incAmount = formData['{}-incAmount'.format(number)]
		setup = cls(startValue = startValue, incAmount = incAmount)
		setup.parseCommonForm(user, formData, number, groups, teamDataSetup)
		return setup

class CheckboxSetup(DataSetup):
	options = models.TextField() #expected to be ':' delineated for options, each option has a long name and then a short name, delineated by ","
	optionsList = []

	def parseOptions(self):
		if not self.optionsList:
			tempList = self.options.split(":")
			for option in tempList:
				option.strip()

				optionLN = optionParts[0].strip() #Long Name
				optionSN = optionParts[1].strip() #Short Name
				optionsList.append([optionLN, optionSN])

		return self.optionsList

	@classmethod
	def createFromForm(cls, user, formData, number, groups, teamDataSetup):
		options = formData['{}-options'.format(number)]
		setup = cls(options = options)
		setup.parseCommonForm(user, formData, number, groups, teamDataSetup)
		return setup

class RadioSetup(DataSetup):
	options = models.TextField() #expected to be ':' delineated for options, each option has a long name and then a short name, delineated by ","
	optionsList = []

	def parseOptions(self):
		if not self.optionsList:
			tempList = self.options.split(':')

			for option in tempList:
				option.strip()
				optionParts = option.split(',')

				if len(optionsParts) < 2:
					#something is wrong, skip
					continue

				optionLN = optionParts[0].strip() #Long Name
				optionSN = optionParts[1].strip() #Short Name
				optionsList.append([optionLN, optionSN])

		return self.optionsList

	@classmethod
	def createFromForm(cls, user, formData, number, groups, teamDataSetup):
		options = formData['{}-options'.format(number)]
		setup = cls(options = options)
		setup.parseCommonForm(user, formData, number, groups, teamDataSetup)
		return setup

#Only Integers
class IntegerSetup(DataSetup):

	@classmethod
	def createFromForm(cls, user, formData, number, groups, teamDataSetup):
		setup = cls()
		setup.parseCommonForm(user, formData, number, groups, teamDataSetup)
		return setup


#Any type of number
class NumberSetup(DataSetup):

	@classmethod
	def createFromForm(cls, user, formData, number, groups, teamDataSetup):
		setup = cls()
		setup.parseCommonForm(user, formData, number, groups, teamDataSetup)
		return setup


###############################################
#The Data
class DataModel(models.Model):
	typeOfData = models.CharField(max_length=20)
	matchRobot = models.ForeignKey(MatchRobot)

	def __unicode__(self):
		return '{} data for {}'.format(this.typeOfData.longName, this.matchRobot.__unicode__())

class IncrementorData(DataModel):
	data = models.IntegerField()
	setup = models.ForeignKey(IncrementorSetup)

	def getEditingHTML(self):
		self.setup.getHTML(True, self.data)

class CheckboxData(DataModel):
	data = models.TextField()
	setup = models.ForeignKey(CheckboxSetup)

	def getEditingHTML(self):
		self.setup.getHTML(True, self.data)

class RadioData(DataModel):
	data = models.TextField()
	setup = models.ForeignKey(RadioSetup)

	def getEditingHTML(self):
		self.setup.getHTML(True, self.data)

class IntegerData(DataModel):
	data = models.IntegerField()
	setup = models.ForeignKey(IntegerSetup)

	def getEditingHTML(self):
		self.setup.getHTML(True, self.data)

class NumberData(DataModel):
	data = models.DecimalField(max_digits = 8, decimal_places=2)
	setup = models.ForeignKey(NumberSetup)

	def getEditingHTML(self):
		self.setup.getHTML(True, self.data)
